/*
 *   Copyright (c) International Business Machines  Corp., 2002
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * linux/drivers/md/syncio.h
 *
 * Provides synchronous I/O support
 */

#ifndef __SYNCIO_H_
#define __SYNCIO_H_

#include <linux/list.h>
#include <linux/slab.h>
#include <linux/wait.h>

/* 
 * The syncio component allows users to call sync_io_setup() to obtain a handle.
 * Attached to the syncio handle is a private slab cache for IO's related structs.
 * For 2.4.x kernel, buffer heads are allocated from this private slab cache.
 *
 * The user of syncio service can share a single syncio handle among its instances.
 * For example, on the creation of the first BBR device, BBR can obtain a 
 * syncio handle and use it for all future BBR instances.  Of course, the code 
 * handles the destruction of the last BBR instance should call sync_io_cleanup().
 */

struct sync_io_handle {
	struct list_head	syncio_list;
	kmem_cache_t		*cachep;
	atomic_t		waiters;
	wait_queue_head_t	cache_wait_queue;
};

struct sync_io_job {
	struct sync_io_handle	*handle;
	kdev_t 			dev;
	int 			rw;
	u64			start_lsn;
	u64			num_lsns;
	void			*data;
};


extern int sync_io_setup(struct sync_io_handle **handle, u8 *name);
extern void sync_io_cleanup(struct sync_io_handle *handle);
extern int sync_io(struct sync_io_job *job);

#endif
