/*
 *  Force feedback support for hid devices.
 *  Not all hid devices use the same protocol. For example, some use PID,
 *  other use their own proprietary procotol.
 *
 *  Copyright (c) 2002 Johann Deneux
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Should you need to contact me, the author, you can do so by
 * e-mail - mail your message to <deneux@ifrance.com>
 */

#include <linux/input.h>

#define DEBUG
#include <linux/usb.h>

#include "hid.h"

/* Drivers' initializing functions */
extern int hid_lgff_init(struct hid_device* hid);
extern int hid_pid_init(struct hid_device* hid);

/* Each hid force feedback driver must provide an init function.
   It must return 1 if the driver does not handle this device,
   0 if it could and the initialization itself went smoothly,
   <0 if the driver should have been able to handle the device, but failed
*/
static int (*inits[])(struct hid_device*) = {
#ifdef CONFIG_LOGITECH_FF
	hid_lgff_init,
#endif
#ifdef CONFIG_HID_PID
	hid_pid_init,
#endif
	NULL
};

int hid_ff_init(struct hid_device* hid)
{
	int i;
	int status;

	for (i=0; inits[i]; ++i) {
		switch (status = inits[i](hid)) {
		case 0: return 0;  // Device handled and init ok
		case 1: break;     // Device not handled
		default: return status; // Device handled, init failed
		}
	}

	dbg("hid_ff_init could not find initializer");
	return -ENOSYS;
}
