#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/poll.h>
#include <linux/string.h>
#include <linux/file.h>
#include <linux/types.h>
#include <linux/grinternal.h>

extern ssize_t write_grsec_handler(struct file * file, const char * buf,
				   size_t count, loff_t *ppos);

static DECLARE_WAIT_QUEUE_HEAD(learn_wait);

char learn_buffer[16384];
ssize_t learn_buffer_len;

static ssize_t
read_learn(struct file *file, char * buf, size_t count, loff_t * ppos)
{
	DECLARE_WAITQUEUE(wait, current);
	ssize_t retval;

	add_wait_queue(&learn_wait, &wait);
	set_current_state(TASK_INTERRUPTIBLE);

	do {
		spin_lock(&grsec_learn_lock);
		if (learn_buffer_len)
			break;
		spin_unlock(&grsec_learn_lock);
		if (file->f_flags & O_NONBLOCK) {
			retval = -EAGAIN;
			goto out;
		}
		if (signal_pending(current)) {
			retval = -ERESTARTSYS;
			goto out;
		}

		schedule();
	} while (1);

	if (count < learn_buffer_len) {
		spin_unlock(&grsec_learn_lock);
		return -EINVAL;
	}

	if (copy_to_user(buf, &learn_buffer, learn_buffer_len))
		retval = -EFAULT;
	else {
		retval = learn_buffer_len;
		learn_buffer_len = 0;
		memset(learn_buffer, 0, sizeof(learn_buffer));
	}

	spin_unlock(&grsec_learn_lock);
out:
	set_current_state(TASK_RUNNING);
	remove_wait_queue(&learn_wait, &wait);
	return retval;
}

static unsigned int
poll_learn(struct file * file, poll_table * wait)
{
	poll_wait(file, &learn_wait, wait);

	if (learn_buffer_len)
		return (POLLIN | POLLRDNORM);
	else
		return 0;
}

void
gr_add_learn_entry(const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	learn_buffer_len = vsnprintf(learn_buffer, sizeof(learn_buffer), fmt, args);
	va_end(args);

	if (learn_buffer_len < 0)
		learn_buffer_len = sizeof(learn_buffer);

	wake_up_interruptible(&learn_wait);

	return;
}

struct file_operations grsec_fops = {
	read:		read_learn,
	write:		write_grsec_handler,
	poll:		poll_learn,
};
