#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/grinternal.h>

void
gr_handle_ioperm(void)
{
	security_alert(GR_IOPERM_MSG, GR_IOPERM_FLD, DEFAULTSECARGS);
	return;
}

void
gr_handle_iopl(void)
{
	security_alert(GR_IOPL_MSG, GR_IOPL_FLD, DEFAULTSECARGS);
	return;
}

void
gr_handle_mem_write(void)
{
	security_alert(GR_MEM_WRITE_MSG, GR_MEM_WRITE_FLD, DEFAULTSECARGS);
	return;
}

void
gr_handle_kmem_write(void)
{
	security_alert(GR_KMEM_MSG, GR_KMEM_FLD, DEFAULTSECARGS);
	return;
}

void
gr_handle_open_port(void)
{
	security_alert(GR_PORT_OPEN_MSG, GR_PORT_OPEN_FLD, DEFAULTSECARGS);
	return;
}

int
gr_handle_mem_mmap(const unsigned long offset, struct vm_area_struct *vma)
{
	if (offset < __pa(high_memory) &&
	    (pgprot_val(vma->vm_page_prot) & PROT_WRITE) &&
	    (offset != 0xa0000 || ((vma->vm_end - vma->vm_start) > 0x20000))) {
		security_alert(GR_MEM_MMAP_MSG, GR_MEM_MMAP_FLD,
			       DEFAULTSECARGS);
		return -EPERM;
	} else if (offset < __pa(high_memory))
		vma->vm_flags &= ~VM_MAYWRITE;

	return 0;
}
