/* This is a module which is used for setting up fake conntracks
 * on packets so that they are not seen by the conntrack/NAT code.
 */
#include <linux/module.h>
#include <linux/skbuff.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	struct ip_conntrack_protocol *proto;
	struct ip_conntrack_tuple tuple;

	if ((*pskb)->nfct != NULL)
		return IPT_CONTINUE;

	proto = ip_ct_find_proto((*pskb)->nh.iph->protocol);

	if (ip_conntrack_get_tuple((*pskb)->nh.iph, (*pskb)->len, 
		      &tuple, proto)
	    && !ip_conntrack_tuple_taken(&tuple, NULL)) {
		(*pskb)->nfct = &ip_conntrack_untracked.infos[IP_CT_NEW];
		nf_conntrack_get((*pskb)->nfct);
	}

	return IPT_CONTINUE;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	if (targinfosize != 0) {
		printk(KERN_WARNING "NOTRACK: targinfosize %u != 0\n",
		       targinfosize);
		return 0;
	}

	if (strcmp(tablename, "raw") != 0) {
		printk(KERN_WARNING "NOTRACK: can only be called from \"raw\" table, not \"%s\"\n", tablename);
		return 0;
	}

	return 1;
}

static struct ipt_target ipt_notrack_reg = { 
	.list = { NULL, NULL }, 
	.name = "NOTRACK", 
	.target = target, 
	.checkentry = checkentry, 
	.destroy = NULL, 
	.me = THIS_MODULE 
};

static int __init init(void)
{
	if (ipt_register_target(&ipt_notrack_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_notrack_reg);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
